@@ -32,6 +32,9 @@
 
 #include <exo/exo.h>
 #include <thunar-archive-plugin/tap-backend.h>
+#ifdef GDK_WINDOWING_WAYLAND
+#include <gdk/gdkwayland.h>
+#endif
 
 
 
@@ -389,6 +392,7 @@ tap_backend_run (const gchar *action,
   GAppInfo                 *mime_application;
   gchar                    *mime_type;
   GdkScreen                *screen;
+  GdkDisplay               *display;
   gchar                    *wrapper;
   gchar                   **argv;
   gchar                   **envp;
@@ -396,7 +400,7 @@ tap_backend_run (const gchar *action,
   GList                    *lp;
   GPid                      pid = -1;
   gint                      n;
-  gchar                    *display = NULL;
+  const gchar              *displayname;
 
   /* determine the mime infos on-demand */
   if (G_LIKELY (content_types == NULL))
@@ -444,9 +448,21 @@ tap_backend_run (const gchar *action,
 
           if (screen != NULL)
             {
-              display = gdk_screen_make_display_name (screen);
-              if (display != NULL)
-                envp = g_environ_setenv(envp, "DISPLAY", display, TRUE);
+              display = gdk_screen_get_display (screen);
+              displayname = gdk_display_get_name (display);
+              if (displayname != NULL)
+                {
+#ifdef GDK_WINDOWING_WAYLAND
+                  if (GDK_IS_WAYLAND_DISPLAY (display))
+                    {
+                      envp = g_environ_setenv (envp, "WAYLAND_DISPLAY", displayname, TRUE);
+                    }
+                  else
+#endif
+                  if (TRUE) {
+                      envp = g_environ_setenv(envp, "DISPLAY", displayname, TRUE);
+                    }
+                }
             }
 
           /* try to run the command */
@@ -456,7 +472,6 @@ tap_backend_run (const gchar *action,
           /* cleanup */
           g_strfreev (envp);
           g_strfreev (argv);
-          g_free (display);
         }
 
       /* cleanup */
